{*******************************************************}
{                                                       }
{                   Avrora  PISHTI                      }
{                                                       }
{           Copyright (c) 1999 Avrora LLC               }
{                                                       }
{*******************************************************}
{ This is PROPRIETARY SOURCE CODE of Avrora LLC;        }
{ the contents of this file may not be disclosed to     }
{ third parties, copied or duplicated in any form, in   }
{ whole or in part, without the prior written           }
{ permission of Avrora LLC.                             }
{*******************************************************}

unit UMainForm;

interface

uses
  Windows, Messages, Classes, Graphics, Controls, Forms,  StdCtrls,
  Menus, ESGraphics, ESCard, ESGameDlg, SysUtils, Placemnt, Digits,
  ESDigits, ESNetManager;

type
  TMainForm = class(TForm)
    MainMenu: TMainMenu;
    MenuGame: TMenuItem;
    MenuStart: TMenuItem;
    MenuLine1: TMenuItem;
    MenuScore: TMenuItem;
    MenuDecks: TMenuItem;
    MenuSettings: TMenuItem;
    MenuLine2: TMenuItem;
    MenuExit: TMenuItem;
    MenuHelp: TMenuItem;
    MenuContents: TMenuItem;
    MenuLine4: TMenuItem;
    MenuAbout: TMenuItem;
    Computer: TESCardGroup;
    Player: TESCardGroup;
    Table: TESCardTable;
    DecksDialog: TESDecksDialog;
    PlayerPishti: TESCardGroup;
    ComputerPishti: TESCardGroup;
    MenuAutoPlay: TMenuItem;
    Sprite: TESSimpleSprite;
    ScoreDialog: TESScoreDialog;
    FormStorage: TFormStorage;
    Deck: TESCardDeck;
    ComputerScore: TESDigits;
    PlayerScore: TESDigits;
    procedure MenuDecksClick(Sender: TObject);
    procedure MenuScoreClick(Sender: TObject);
    procedure MenuContentsClick(Sender: TObject);
    procedure MenuAboutClick(Sender: TObject);
    procedure MenuExitClick(Sender: TObject);
    procedure PlayerCardClick(Sender: TESCardGroup; Card: TESSingleCard;
      Position: Integer; var PlayIt: Boolean; var TablePos: TESTablePos);
    procedure MenuStartClick(Sender: TObject);
    procedure TableAfterPartyFinish(Sender: TESCardTable);
    procedure ComputerBeforePlay(Sender: TESCardGroup;
      var Position: Integer; var TablePos: TESTablePos);
    procedure TableAfterDealToTable(Sender: TESCardTable; PlayedCardsCount,
      TopCardPos: Integer; TopCard: TESSingleCard);
    procedure MenuAutoPlayClick(Sender: TObject);
    procedure TableAfterAddCard(Sender: TESCardTable; PlayedCardsCount,
      TopCardPos: Integer; TopCard: TESSingleCard; TopCardID,
      PrevCardID: Integer; OwnedCard: Boolean);
    procedure FormActivate(Sender: TObject);
    procedure MenuSettingsClick(Sender: TObject);
    procedure TableAfterTrickFinish(Sender: TESCardTable);
    procedure PlayerScoreRotateEnd(Sender: TObject);
    procedure PlayerAfterScoreChange(Sender: TESCardGroup);
    procedure ComputerAfterScoreChange(Sender: TESCardGroup);
    procedure ComputerScoreRotateEnd(Sender: TObject);
    procedure MenuGameClick(Sender: TObject);
    procedure TableAfterDealingFinish(Sender: TESCardTable);
    procedure MenuSortedClick(Sender: TObject);
  private
    { Private declarations }
    TableShow : Boolean;
    procedure NewGame;
    procedure ShowTable;
    procedure BadDeck;
    function LeastRiskCard(Sender : TESCardGroup) : Integer;
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

uses UDialogAbout, UDialogTable, UDialogSettings, Dialogs, UHelpForm;

{$R *.DFM}

{ MENU EVENT HANDLERS }

procedure TMainForm.FormActivate(Sender: TObject);
begin
  Update;
  NewGame;
end;

procedure TMainForm.MenuGameClick(Sender: TObject);
begin
  if Sprite.Busy then
    MenuStart.Caption := '&Stop'
  else
    MenuStart.Caption := '&Start'
end;

procedure TMainForm.MenuAutoPlayClick(Sender: TObject);
begin
  TMenuItem(Sender).Checked := not TMenuItem(Sender).Checked;
  if TMenuItem(Sender).Checked then
  begin
    Player.BeforePlay := ComputerBeforePlay;
    Player.Controller := ctComputer;
    Table.StartPlay;
  end
  else
  begin
    Player.Controller := ctHuman;
    Player.BeforePlay := nil;
  end;
end;

procedure TMainForm.MenuStartClick(Sender: TObject);
begin
  NewGame;
end;

procedure TMainForm.MenuExitClick(Sender: TObject);
begin
  Close;
end;

procedure TMainForm.MenuDecksClick(Sender: TObject);
begin
  DecksDialog.Execute;
end;

procedure TMainForm.MenuScoreClick(Sender: TObject);
begin
  ScoreDialog.Execute;
end;

procedure TMainForm.MenuContentsClick(Sender: TObject);
begin
  Application.ProcessMessages;
  with THelpForm.Create(Application) do
  try
    ShowModal;
  finally
    Free;
  end;
end;

procedure TMainForm.MenuAboutClick(Sender: TObject);
begin
  Application.ProcessMessages;
  with TDialogAbout.Create(Application) do
  try
    ShowModal;
  finally
    Free;
  end;
end;

procedure TMainForm.MenuSettingsClick(Sender: TObject);
begin
  with TDialogSettings.Create(Application) do
  try
    FinishScore.Text := IntToStr(Table.FinishScore);
    AnimSpeed.Position := Sprite.IncrementY;
    DialogDelay.Value := ScoreDialog.Tag;
    TableDelay.Value := Table.TakeDelay;
    DScore.Checked := PlayerScore.Visible;
    DPoints.Checked := (Table.Tag = 1);
    RDeck.Checked := Table.RandomDeck;
    CDeck.Checked := Table.ChangeDeck;
    ShowModal;
    if ModalResult = mrOk then
    begin
      Table.FinishScore := Trunc(FinishScore.Value);
      Sprite.IncrementY := AnimSpeed.Position;
      Sprite.IncrementX := Sprite.IncrementY * 2;
      ScoreDialog.Tag := Trunc(DialogDelay.Value);
      Table.TakeDelay := Trunc(TableDelay.Value);
      PlayerScore.Value :=Player.Score;
      PlayerScore.Visible := DScore.Checked;
      ComputerScore.Value := Computer.Score;
      ComputerScore.Visible := DScore.Checked;
      Table.RandomDeck := RDeck.Checked;
      Table.ChangeDeck := CDeck.Checked;
      if DPoints.Checked then
        Table.Tag := 1
      else
        Table.Tag := 0;
    end;
  finally
    Free;
  end;
end;

{ GAME MANAGEMENT }

procedure TMainForm.PlayerCardClick(Sender: TESCardGroup;
  Card: TESSingleCard; Position: Integer; var PlayIt: Boolean;
  var TablePos: TESTablePos);
begin
  PlayIt := True;
end;

procedure TMainForm.TableAfterPartyFinish(Sender: TESCardTable);
begin

  { KAGIT 3 }

  if Player.TakenCardsCount > Computer.TakenCardsCount then
    Player.Score := Player.Score + 3
  else
  if Player.TakenCardsCount < Computer.TakenCardsCount then
    Computer.Score := Computer.Score + 3;

  { PISTILER }

  ComputerScore.Value := Computer.Score;
  PlayerScore.Value := Player.Score;

  ScoreDialog.Update;
  if Table.Tag = 1 then
  begin
    Sender.ShowTakenCards;
    ScoreDialog.Execute;
    Sender.HideTakenCards;
  end;
  Sender.Clear(True);
  PlayerPishti.CardList.Clear;
  ComputerPishti.CardList.Clear;

  { BERABERLIK VAR MI? }

  if (Player.Score >= (Sender.FinishScore + Sender.FinishScoreDelta)) and
     (Computer.Score >= (Sender.FinishScore+ Sender.FinishScoreDelta)) then
     if Player.Score = Computer.Score then
       Sender.FinishScoreDelta := Sender.FinishScoreDelta + 10;

  { OYUN SONU GELDI MI? }

  if not ((Player.Score >= (Sender.FinishScore + Sender.FinishScoreDelta)) or
     (Computer.Score >= (Sender.FinishScore+ Sender.FinishScoreDelta))) then
    begin
      TableShow := True;
      Sender.PrepareDeck;
      Table.NextCardPos := 0;
      Sender.DealToTable;
      Sender.Deal(True);
    end;
end;


procedure TMainForm.TableAfterDealToTable(Sender: TESCardTable;
  PlayedCardsCount, TopCardPos: Integer; TopCard: TESSingleCard);
begin
  { CHECK FOR VALETS IN THE TABLE }
  if TopCard.Value = 11 then
  begin
    Sender.TopCardID := Sender.Cards[2].ID;
    Sender.Cards[2].ShowBack := False;
    if Sender.Cards[2].Value = 11 then
    begin
      Sender.TopCardID := Sender.Cards[1].ID;
      Sender.Cards[1].ShowBack := False;
    end;
    if Sender.Cards[1].Value = 11 then
    begin
      Sender.TopCardID := Sender.Cards[0].ID;
      Sender.Cards[0].ShowBack := False;
    end;
  end;
end;

procedure TMainForm.TableAfterAddCard(Sender: TESCardTable;
  PlayedCardsCount, TopCardPos: Integer; TopCard: TESSingleCard; TopCardID,
  PrevCardID: Integer; OwnedCard: Boolean);
var
  I : Integer;
begin
  { TABLE RULES }

  if OwnedCard then
  begin
    if GetCardValue(TopCardID) = GetCardValue(PrevCardID) then
    begin
      if (PlayedCardsCount = 2) and (TopCard.Value <> 11) and (Sender.RemainingCardsCount > 0) then
      begin
        if TopCard.CardOwner = Player then
        begin
          with PlayerPishti.CardList.Add do
            Card.ID := TopCard.ID;
          Player.Score := Player.Score + 10;
        end
        else
        begin
           with ComputerPishti.CardList.Add do
             Card.ID := TopCard.ID;
           Computer.Score := Computer.Score + 10;
        end
      end;
      Sender.SendTableTo(TopCard.CardOwner);
      
      if (TopCard.CardOwner = Player) and TableShow then
        ShowTable
      else
        TableShow := False;

      if (TopCard.CardOwner = Computer) and TableShow then
        for I := 0 to 2 do
          Table.AddKnownID(Computer.TakenCards[I])
      else
        TableShow := False;
    end
    else
    if (TopCard.Value = 11) and (PlayedCardsCount > 1) then
    begin
      Sender.SendTableTo(TopCard.CardOwner);
      if (TopCard.CardOwner = Player) and TableShow then
        ShowTable
      else
        TableShow := False;
    end;
  end;

end;

procedure TMainForm.NewGame;
begin
  if Sprite.Busy then
    Sprite.Stop;
  if not Table.Busy then
  begin
    Table.FinishScoreDelta := 0;
    TableShow := True;
    PlayerPishti.CardList.Clear;
    ComputerPishti.CardList.Clear;

    ScoreDialog.Clear;
    Table.Clear(True);
    Table.PrepareDeck;
    Table.NextCardPos := 0;
    Table.DealToTable;
    Table.Deal(True);
  end;
end;

procedure TMainForm.ShowTable;
var
  I : Integer;
begin
  if TableShow then
  begin
    TableShow := False;
    with TDialogTable.Create(Application) do
    try
      for I := 0 to 3 do
        TableCards.Cards[I].ID := Player.TakenCards[I];
      Timer.Interval := ScoreDialog.Tag;
      ShowModal;
    finally
      Free;
    end;
  end;
end;

procedure TMainForm.TableAfterTrickFinish(Sender: TESCardTable);
begin
  if Sender.LastTrick then
    Sender.SendTableTo(Sender.LastTaker)
  else
    Sender.Deal(False);
end;

procedure TMainForm.PlayerAfterScoreChange(Sender: TESCardGroup);
begin
  if PlayerScore.Visible then
  if Player.Score <> 0 then
    PlayerScore.Increase
  else
    PlayerScore.Value := 0;
end;

procedure TMainForm.PlayerScoreRotateEnd(Sender: TObject);
begin
  if Abs(Player.Score - PlayerScore.Value) >= 4 then
    PlayerScore.IncDecSteps := 4
  else
    PlayerScore.IncDecSteps := 10;
  if PlayerScore.Value < Player.Score then
    PlayerScore.Increase;
end;

procedure TMainForm.ComputerAfterScoreChange(Sender: TESCardGroup);
begin
  if ComputerScore.Visible then
  if Computer.Score <> 0 then
    ComputerScore.Increase
  else
    ComputerScore.Value := 0;
end;

procedure TMainForm.ComputerScoreRotateEnd(Sender: TObject);
begin
  if Abs(Computer.Score - ComputerScore.Value) >= 4 then
    ComputerScore.IncDecSteps := 4
  else
    ComputerScore.IncDecSteps := 10;
  if ComputerScore.Value < Computer.Score then
    ComputerScore.Increase;
end;

procedure TMainForm.TableAfterDealingFinish(Sender: TESCardTable);
var
  I, A, J : Integer;
begin
  J := 1;
  A := Player.Cards[0].Value;
  for I := 1 to 3 do
    if Player.Cards[I].Value = A then
      Inc(J);

  if J = 4 then
  begin
    BadDeck;
    Exit;
  end;

  J := 1;
  A := Computer.Cards[0].Value;
  for I := 1 to 3 do
    if Computer.Cards[I].Value = A then
      Inc(J);

  if J = 4 then
  begin
    BadDeck;
    Exit;
  end;

  J := 1;
  A := Table.Cards[0].Value;
  for I := 1 to 3 do
    if Table.Cards[I].Value = A then
      Inc(J);

  if J = 4 then
  begin
    BadDeck;
    Exit;
  end;
end;

procedure TMainForm.BadDeck;
var
  I : Integer;
begin
  Exit;
  Computer.ShowBack := False;
  for I := 0 to 3 do
    Table.Cards[I].ShowBack := False;
  ShowMessage('All the 4 cards are same, trick will restart.');
  Computer.ShowBack := True;
  Player.Score := Player.Score - Player.TakenCardsPoints - PlayerPishti.CardsCount * 10;
  Computer.Score := Computer.Score - Computer.TakenCardsPoints - ComputerPishti.CardsCount * 10;
  ComputerScore.Value := Computer.Score;
  PlayerScore.Value := Player.Score;
  TableShow := True;
  PlayerPishti.CardList.Clear;
  ComputerPishti.CardList.Clear;
  Table.Clear(True);
  Table.PrepareDeck;
  Table.DealToTable;
  Table.Deal(True);
end;

procedure TMainForm.MenuSortedClick(Sender: TObject);
begin
  TMenuItem(Sender).Checked := not TMenuItem(Sender).Checked;
  if TMenuItem(Sender).Checked then
    Player.SortStyle := soValue
  else
    Player.SortStyle := soNone;
end;

{ COMPUTER AI }

procedure TMainForm.ComputerBeforePlay(Sender: TESCardGroup;
  var Position: Integer; var TablePos: TESTablePos);
var
  A : Integer;
begin
  for A := 0 to Sender.CardsCount - 1 do
  begin
    Sender[A].Tag := 0;
    Sender[A].Tag := Table.KnownValues[Sender[A].Value] + 1;
  end;
  A := Sender.FindValuePos(GetCardValue(Table.PlayedCards[Table.PlayedCardsCount - 1]));
  if (A >=0) and (A <> 11) then      { THE CARD EXISTS IN HAND AND IT IS NOT A VALET }
    Position := A
  else
  begin
    A := Sender.FindValuePos(GetCardValue(Table.PlayedCards[Table.PlayedCardsCount - 2]));
    if (A >=0) and  (Sender.NonEmptyCardsCount <> 4) then    { DEFENSE : PLAY PREVIOUS CARD IF POSSIBLE }
    begin
      Position := A;
//      ShowMessage('PrevCard');
    end
    else
    begin              { DEFENSE : PLAY LEAST RISK CARD }
      A := LeastRiskCard(Sender);
      if A >= 0 then
      begin
        Position := A;
//        ShowMessage('LeastRisk');
      end
      else
      begin             { CHECK FOR VALET }
        A := Sender.FindValuePos(11);
        if (A >=0) and (Table.PlayedCardsCount > 0) then
        begin
          Position := A;
//          ShowMessage('Valet');
        end
      end;
    end;
  end
end;

function TMainForm.LeastRiskCard(Sender: TESCardGroup): Integer;
var
  I, J : Integer;
  Max : Integer;
  Pos : Integer;
begin
  Result := -1;
  Max := -1;
  Pos := -1;

  { INCREASE TAG OF SAME CARDS }
  for I := 0 to Sender.NonEmptyCardsCount - 2 do
    for J := I + 1 to Sender.NonEmptyCardsCount - 1 do
      if Sender.NonEmptyCards[I].Value = Sender.NonEmptyCards[J].Value then
        Sender.NonEmptyCards[I].Tag := Sender.NonEmptyCards[I].Tag + 1;

  for I := 0 to Sender.NonEmptyCardsCount - 1 do
    if (Sender.NonEmptyCards[I].Tag > Max) and (Sender.NonEmptyCards[I].Value <> 11) then
    begin
      Max := Sender.NonEmptyCards[I].Tag;
      Pos := Sender.NonEmptyCards[I].Position;
    end;

  if not Table.LastTrick then
  begin
    if Max >= 2 then
      Result := Pos;
  end
  else
    if Max >= 4 then
      Result := Pos
end;

end.
