{*******************************************************}
{                                                       }
{         Avrora Card Gaming Components Demo            }
{                                                       }
{             Written by Avrora LLC  1999               }
{                                                       }
{*******************************************************}
{ This demo shows how easy prototyping a game will be   }
{ with Avrora Card Gaming Components.  The only thing   }
{ left for you is writing the game AI and enforcing     }
{ additional game rules which are specific to your game.}
{                                                       }
{ This demo uses just 5 specific event handlers and     }
{ a few properties. Manual written code < 50 lines.     }
{ (Rules are opposite of Hearts)                        }
{*******************************************************}
{ You can freely modify and distribute this program.    }
{*******************************************************}


unit UMainForm;

interface

uses
  Windows, Messages, Menus, Classes, Graphics, Controls, Forms,
  ESGraphics, ESCard, ESGameDlg, StdCtrls, Dialogs, SYsutils;

type
  TDemoForm = class(TForm)
    Player: TESCardGroup;
    Computer1: TESCardGroup;
    Computer2: TESCardGroup;
    Computer3: TESCardGroup;
    Table: TESCardTable;
    Sprite: TESSimpleSprite;
    DecksDialog: TESDecksDialog;
    MainMenu: TMainMenu;
    MenuGame: TMenuItem;
    MenuStart: TMenuItem;
    MenuSep1: TMenuItem;
    MenuDealAnimation: TMenuItem;
    MenuSep2: TMenuItem;
    MenuDecks: TMenuItem;
    MenuScore: TMenuItem;
    MenuSep3: TMenuItem;
    MenuExit: TMenuItem;
    MenuAutoPlay: TMenuItem;
    MenuShowAll: TMenuItem;
    ScoreDialog: TESScoreDialog;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    Deck: TESCardDeck;
    procedure PlayerCardClick(Sender: TESCardGroup; Card: TESSingleCard;
      Position: Integer; var PlayIt: Boolean; var TablePos: TESTablePos);
    procedure ComputerBeforePlay(Sender: TESCardGroup;
      var Position: Integer; var TablePos: TESTablePos);
    procedure TableAfterPartyFinish(Sender: TESCardTable);
    procedure MenuStartClick(Sender: TObject);
    procedure MenuDecksClick(Sender: TObject);
    procedure MenuScoreClick(Sender: TObject);
    procedure MenuExitClick(Sender: TObject);
    procedure MenuDealAnimationClick(Sender: TObject);
    procedure MenuAutoPlayClick(Sender: TObject);
    procedure MenuShowAllClick(Sender: TObject);
    procedure TableAfterTrickFinish(Sender: TESCardTable);
    procedure FormActivate(Sender: TObject);
    procedure MenuGameClick(Sender: TObject);
    procedure TableBeforeAddCard(Sender: TESCardTable; Card: TESSinglecard;
      SuitExists: Boolean; var Accept: Boolean);
  private
    { Private declarations }
    procedure NewGame;
  public
    { Public declarations }
  end;

var
  DemoForm: TDemoForm;

implementation

{$R *.DFM}

{ SPECIFIC EVENT HANDLERS }

{1. OnCardClick : Here we will determine what will happen if human controlled
                  player clicks a card. }

procedure TDemoForm.PlayerCardClick(Sender: TESCardGroup;
  Card: TESSingleCard; Position: Integer; var PlayIt: Boolean;
  var TablePos: TESTablePos);
begin
  PlayIt := True;
end;

{2. BeforePlay : Here we will determine which card the computer controlled
                 players will play.  All computer controlled players' event handler
                 points to this same one. }

procedure TDemoForm.ComputerBeforePlay(Sender: TESCardGroup;
  var Position: Integer; var TablePos: TESTablePos);
var
  A, B : Integer;
begin
  { Just a very basic AI. }
  A := Sender.FindMaxSuitPos(Table.Suit);
  B := Sender.FindMinSuitPos(Table.Trump);
  Position := Sender.FindMaxSuitPosInvert(Table.Trump);
  if (A >= 0) and (Table.Suit <> stJoker) then
    Position := A
  else
  if (Table.SuitBroken and (B >=0)) or ((Table.Suit <>stJoker) and (B >=0)) then
    Position := B;
end;

{3. AfterTrickFinish : Here we will determine what will happen after the trick finishes.}

procedure TDemoForm.TableAfterTrickFinish(Sender: TESCardTable);
begin
  Sender.SendTableTo(Sender.BiggestCardOwner);
end;

{4. AfterPartyFinish : Here we will determine what will happen after the party finishes.}

procedure TDemoForm.TableAfterPartyFinish(Sender: TESCardTable);
begin
  Label9.Caption := '';
  Sender.ShowTakenCards;
  ScoreDialog.Update;
  ScoreDialog.Execute;
  Sender.HideTakenCards;
  Sender.Clear(True);
  Sender.PrepareDeck;
  Sender.Deal(True);
end;

{5. BeforeAddCard : Here we will show whether the strong suit has broken .}

procedure TDemoForm.TableBeforeAddCard(Sender: TESCardTable;
  Card: TESSinglecard; SuitExists: Boolean; var Accept: Boolean);
begin
  if Sender.SuitBroken then
    Label9.Caption := 'Hearts Broken.';
end;

{ MENU EVENT HANDLERS }

procedure TDemoForm.MenuGameClick(Sender: TObject);
begin
  if Sprite.Busy then
    MenuStart.Caption := '&Stop'
  else
    MenuStart.Caption := '&Start';
end;

procedure TDemoForm.NewGame;
begin
  if Sprite.Busy then
    Sprite.Stop;
  if not Table.Busy then
  begin
    Label9.Caption := '';
    ScoreDialog.Clear;
    Table.Clear(True);
    Table.PrepareDeck;
    Table.Deal(True);
  end;
end;

procedure TDemoForm.MenuStartClick(Sender: TObject);
begin
  NewGame;
end;

procedure TDemoForm.MenuDealAnimationClick(Sender: TObject);
begin
  TMenuItem(Sender).Checked := not TMenuItem(Sender).Checked;
  Table.DealAnimation := TMenuItem(Sender).Checked;
end;

procedure TDemoForm.MenuDecksClick(Sender: TObject);
begin
  DecksDialog.Execute;
end;

procedure TDemoForm.MenuScoreClick(Sender: TObject);
begin
  ScoreDialog.Execute;
end;

procedure TDemoForm.MenuExitClick(Sender: TObject);
begin
  Close;
end;

procedure TDemoForm.MenuAutoPlayClick(Sender: TObject);
begin
  Application.ProcessMessages;
  TMenuItem(Sender).Checked := not TMenuItem(Sender).Checked;
  if TMenuItem(Sender).Checked then
  begin
    Player.BeforePlay := ComputerBeforePlay;
    Player.Controller := ctComputer;
    Table.StartPlay;
  end
  else
  begin
    Player.Controller := ctHuman;
    Player.BeforePlay := nil;
  end;
end;

procedure TDemoForm.MenuShowAllClick(Sender: TObject);
begin
  Computer1.ShowBack := TMenuItem(Sender).Checked;
  Computer2.ShowBack := Computer1.ShowBack;
  Computer3.ShowBack := Computer1.ShowBack;
  TMenuItem(Sender).Checked := not TMenuItem(Sender).Checked;
end;

{ FORM'S EVENT HANDLERS }

procedure TDemoForm.FormActivate(Sender: TObject);
begin
  Update; { FORCE THE FORM TO COMPLETE ALL ITS PAINTING. DO NOT FORGET. }
  NewGame;
end;

end.
